package bcn;


public class InputBuffer<T> implements InputPort<T> {
    
    /** Received messages. */
    private final T[] buffer;
    
    /** Position of the oldest received message (if any). */
    private int get;
    
    /** Position of the next free slot (if any). */
    private int set;
    
    
    public InputBuffer(int size) {
        if (size < 1) {
            throw new IllegalArgumentException("Buffer size must be at least 1.");
        }
        buffer = (T[])new Object[size];
        get = set = 0;
    }
    
    
    @Override
    public synchronized T get() {
        T msg = buffer[get];
        if (msg != null) {
            buffer[get] = null;
            get = incIndex(get);
        }
        return msg;
    }
    
    
    @Override
    public synchronized T peek() {
        return buffer[get];
    }
    
    
    @Override
    public synchronized void put(T msg) {
        if (msg == null) {
            throw new IllegalArgumentException("Invalid null message.");
        }
        if (buffer[set] != null) {  // buffer full
            // in this situation 'get' = 'set' and both refers to the oldest message:
            // overwrite the oldest message and move 'get' on the following one
            get = incIndex(get);
        }
        buffer[set] = msg;
        set = incIndex(set);
    }
    
    
    private int incIndex(int index) {
        index++;
        return (index < buffer.length  ?  index  :  0);
        //return (index+1)%buffer.length;
    }
    
}
