package foraging;

import com.cyberbotics.webots.controller.Camera;


public class Detect extends foraging_stub.Detect_stub {
    
	static public class Detected {
		public final float left, front, right;
		
		public Detected (float left, float front, float right) {
			this.left = left;
			this.front = front;
			this.right = right;
		}
	}
	
	
	static private final float RED_RATIO = 2f, DETECTED_THRESHOLD = 0.4f;
    
    private final Camera cam;
    private final int w, h;
    
    
    public Detect(Camera cam) {
        super();
        this.cam = cam;
        w = cam.getWidth();
        h = cam.getHeight();
    }
    
    
    @Override
    public void step() {
        int[] img = cam.getImage();
        float l = leftRedRatio(img);
        float f = middleRedRatio(img);
        float r = rightRedRatio(img);
        if (l < DETECTED_THRESHOLD) {
        	l = 0;
        }
        if (f < DETECTED_THRESHOLD) {
        	f = 0;
        }
        if (r < DETECTED_THRESHOLD) {
        	r = 0;
        }
        if (l + f + r > 0) {
            //System.out.println("Detect:  left = " + l + "  front = " + f + "  right = " + r);
            _detected.send(new Detected(l, f, r));
        }
    }
    
    
    private float leftRedRatio(int[] img) {
        return redRatio(img, w, h, 0, w/3);
    }
    
    
    private float middleRedRatio(int[] img) {
        return redRatio(img, w, h, w/3, w*2/3);
    }
    
    
    private float rightRedRatio(int[] img) {
        return redRatio(img, w, h, w*2/3, w);
    }
    
    
    /** Compute the ratio of almost-red pixels in a portion of an image. */
    private float redRatio(int[] img, int w, int h, int start, int end) {
    	int count = 0;
        for (int x=start; x<end; x++) {  // columns
            for (int y=0; y<h; y++) {  // rows
                int r = Camera.imageGetRed(img, w, x, y);
                int g = Camera.imageGetGreen(img, w, x, y);
                int b = Camera.imageGetBlue(img, w, x, y);
                if (r >= g*RED_RATIO  &&  r >= b*RED_RATIO) {  // the pixel is almost-red
                    count++;
                }
            }
        }
        return ((float)count)/((float)((end-start)*h));
    }
    
}
